@@ -1,10 +1,10 @@
 """Tornado handler for bodo cluster management."""
 
-from email.mime import base
 import json
 
 from jupyter_server.base.handlers import APIHandler
 from jupyter_server.utils import url_path_join
+from jupyter_server.terminal.handlers import TermSocket
 from tornado import web
 
 from .config_vars import should_auto_attach, enable_catalog_mode
@@ -14,8 +14,8 @@ from .remote_ikernels_manager import (
     get_remote_kernel_name_for_cluster,
 )
 from .ssh_keys_manager import cleanup_ssh_keys, get_cluster_ssh_key_fname
-from .config import BASTION_IP, BASTION_USER, CLUSTER_USER
-
+from .config import BASTION_IP, BASTION_USER, CLUSTER_USER, ALLOW_LOCAL_EXECUTION
+from .cluster_terminal import ClusterTerminalRootHandler
 
 class ClusterRemoteIKernelHandler(APIHandler):
     """
@@ -126,54 +126,6 @@ class PlatformClusterListHandler(APIHandler):
         self.finish(json.dumps({"clusters": clusters, "e": error}))
 
 
-class AutoAttachHandler(APIHandler):
-    """
-    Determine whether a cluster should be auto attached to a notebook.
-    """
-
-    @web.authenticated
-    async def get(self):
-        logging_prefix = "[AutoAttachHandler.get]"
-        self.log.info(f"{logging_prefix} Starting...")
-        error = None
-
-        try:
-            auto_attach = should_auto_attach()
-            self.log.info(f"{logging_prefix} auto_attach: {auto_attach}")
-
-        except Exception as e:
-            self.log.error(f"{logging_prefix} Error: {e}")
-            auto_attach = None
-            error = str(e)
-
-        self.log.info(f"{logging_prefix} Finishing...")
-        self.finish(json.dumps({"auto_attach": auto_attach, "e": error}))
-
-
-class CatalogModeHandler(APIHandler):
-    """
-    Determine whether catalog mode should be enabled
-    """
-
-    @web.authenticated
-    async def get(self):
-        logging_prefix = "[CatalogmodeHandler.get]"
-        self.log.info(f"{logging_prefix} Starting...")
-        error = None
-
-        try:
-            catalog_mode = enable_catalog_mode()
-            self.log.info(f"{logging_prefix} catalog_mode: {catalog_mode}")
-
-        except Exception as e:
-            self.log.error(f"{logging_prefix} Error: {e}")
-            catalog_mode = None
-            error = str(e)
-
-        self.log.info(f"{logging_prefix} Finishing...")
-        self.finish(json.dumps({"catalog_mode": catalog_mode, "e": error}))
-
-
 class ResumeClusterHandler(APIHandler):
     @web.authenticated
     async def put(self, cluster_id: str) -> None:
@@ -213,6 +165,42 @@ class PauseClusterHandler(APIHandler):
         self.log.info(f"{logging_prefix} Finishing...")
         self.finish(json.dumps({"e": error}))
 
+class ConfigHandler(APIHandler):
+    @web.authenticated
+    async def get(self) -> None:
+        """
+        Provide config for frontend
+        """
+        logging_prefix = "[ConfigHandler.get]"
+        self.log.info(f"{logging_prefix} Starting...")
+        error = ""
+        try:
+            catalog_mode = enable_catalog_mode()
+            self.log.info(f"{logging_prefix} catalog_mode: {catalog_mode}")
+
+        except Exception as e:
+            self.log.error(f"{logging_prefix} Error: {e}")
+            catalog_mode = None
+            error = str(e)
+
+        try:
+            auto_attach = should_auto_attach()
+            self.log.info(f"{logging_prefix} auto_attach: {auto_attach}")
+
+        except Exception as e:
+            self.log.error(f"{logging_prefix} Error: {e}")
+            auto_attach = None
+            error += str(e)
+        
+        
+        config = {"allowLocalExecution": ALLOW_LOCAL_EXECUTION, "catalog_mode": catalog_mode, "auto_attach": auto_attach}
+        
+        self.log.info(f"{logging_prefix} Finishing...")
+        if error:
+            self.finish(json.dumps({"e": error}))
+            return
+        self.finish(json.dumps(config))
+
 
 class GetCatalogsHandler(APIHandler):
     @web.authenticated
@@ -253,17 +241,23 @@ def setup_handlers(web_app):
         base_url, rf"/bodo/cluster/{cluster_id_regex}/pause"
     )
 
-    auto_attach_path = url_path_join(base_url, r"/bodo/autoattach")
     get_catalogs_path = url_path_join(base_url, r"/bodo/catalog")
-    catalog_mode_path = url_path_join(base_url, r"/bodo/catalogmode")
+
+    # Overriding the default handlers
+    cluster_terminal_root_path = url_path_join(base_url, r"/api/terminals")
+    terminal_socket_path = url_path_join(base_url, r"/terminals/websocket/(\w+)")
+
+    config_path = url_path_join(base_url, r"/bodo/config")
+
 
     handlers = [
         (remote_ikernel_cluster_path, ClusterRemoteIKernelHandler),
         (cluster_list_path, PlatformClusterListHandler),
-        (auto_attach_path, AutoAttachHandler),
         (resume_cluster_path, ResumeClusterHandler),
         (pause_cluster_path, PauseClusterHandler),
+        (cluster_terminal_root_path, ClusterTerminalRootHandler),
+        (terminal_socket_path, TermSocket, {"term_manager": web_app.settings["terminal_manager"]}),
+        (config_path, ConfigHandler),
         (get_catalogs_path, GetCatalogsHandler),
-        (catalog_mode_path, CatalogModeHandler)
     ]
     web_app.add_handlers(".*$", handlers)
