@@ -5,13 +5,16 @@ import { ICluster } from './types';
 import { ReactWidget } from '@jupyterlab/apputils';
 import ClusterResumePause from './resumePauseButton';
 import ClusterStatus from './clusterStatus';
+import ClusterTerminalButton from './clusterTerminal/clusterTerminalButton';
+import { JupyterFrontEnd } from '@jupyterlab/application';
 
 interface IClusterListingItemProps {
   cluster: ICluster;
   forceUpdate: () => void;
+  app: JupyterFrontEnd;
 }
 function ClusterListItem(props: IClusterListingItemProps) {
-  const { cluster, forceUpdate } = props;
+  const { cluster, forceUpdate, app } = props;
   const itemClass = 'bodo-ClusterListingItem';
   const [isProcessing, setProcessing] = React.useState(false);
   const [loadingType, setLoadingType] = React.useState<string>('');
@@ -45,18 +48,21 @@ function ClusterListItem(props: IClusterListingItemProps) {
       {/* <div className="bodo-ClusterListingItem-stats">
         <span className="bodo-ClusterListingItem-stats-name">Cluster ID:</span> {cluster.uuid}
       </div> */}
-      <div className="bodo-ClusterListingItem-status-and-pause">
+      <div className="bodo-ClusterListingItem-status-and-buttons">
         <div className="bodo-ClusterListingItem-stats">
           <span className="bodo-ClusterListingItem-stats-name">Status:</span>{' '}
           <ClusterStatus status={loadingType ? loadingType : cluster.status} />
         </div>
-        <ClusterResumePause
-          forceUpdate={() => forceUpdate()}
-          startLoading={startLoading}
-          loading={isProcessing}
-          uuid={cluster.uuid}
-          clusterStatus={cluster.status}
-        />
+        <div className="bodo-ClusterListingItem-buttons">
+          <ClusterTerminalButton app={app} cluster={cluster} />
+          <ClusterResumePause
+            forceUpdate={() => forceUpdate()}
+            startLoading={startLoading}
+            loading={isProcessing}
+            uuid={cluster.uuid}
+            clusterStatus={cluster.status}
+          />
+        </div>
       </div>
     </li>
   );
@@ -74,15 +80,18 @@ function EmptyClustersList(props: any) {
 
 interface IClusterListProps {
   store: ClusterStore;
+  app: JupyterFrontEnd;
 }
 interface IClusterListState {
   clusters: ICluster[];
 }
 
 export class ClusterList extends React.Component<IClusterListProps, IClusterListState> {
+  private _app: JupyterFrontEnd;
   constructor(props: IClusterListProps) {
     super(props);
     this.state = { clusters: props.store.clusters };
+    this._app = props.app;
   }
 
   componentDidMount() {
@@ -117,7 +126,13 @@ export class ClusterList extends React.Component<IClusterListProps, IClusterList
       );
     if (clusters.length > 0) {
       listing = renderedClusters.map((cluster) => {
-        return <ClusterListItem forceUpdate={() => this.forceUpdate()} cluster={cluster} />;
+        return (
+          <ClusterListItem
+            forceUpdate={() => this.forceUpdate()}
+            cluster={cluster}
+            app={this._app}
+          />
+        );
       });
     } else {
       listing = <EmptyClustersList />;
@@ -136,17 +151,19 @@ export class ClusterList extends React.Component<IClusterListProps, IClusterList
 
 export class BodoClusterListSidebar extends ReactWidget {
   private _store: ClusterStore;
+  private _app: JupyterFrontEnd;
 
-  constructor(store: ClusterStore) {
+  constructor(store: ClusterStore, app: JupyterFrontEnd) {
     super();
     this.addClass('bodo-Sidebar');
     this._store = store;
+    this._app = app;
   }
 
   render() {
     return (
       <div>
-        <ClusterList store={this._store} />
+        <ClusterList store={this._store} app={this._app} />
       </div>
     );
   }
